#from Acquisition import aq_base
#from Acquisition import aq_inner
from urllib import quote_plus
from Products.Five.browser import BrowserView
#from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import shasattr

from plone.memoize.instance import memoize

from zope.interface import implements

from betahaus.openmember.interfaces import IOMBaseView

class OMBaseView(BrowserView):
    """ Base view for OpenMember listings
    """
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.openmember_catalog = getToolByName(self.context, 'openmember_catalog')
        self.omindexes = self.openmember_catalog.indexes()
        self.reference_catalog = getToolByName(self.context, 'reference_catalog')

        #Collect all uids from all indexes that have a UIDLike content so we cache the lookup of objects
        self.membership_uids = []
        for index in self.omindexes:
            if self.openmember_catalog.isUIDLikeIndex(index):
                # This is a index with UIDLike content cache it.
                item = self.openmember_catalog.Indexes.get(index)
                for val in item.uniqueValues():
                    self.membership_uids.append(val)

        #Fetch objects that people in this listing are associated to.
        result = {}
        for uid in self.membership_uids:
            obj = self.reference_catalog.lookupObject(uid)
            if obj:
                result[uid] = obj
        self.referenced_objects = result
        
    def getAddressFields(self):
        portal_properties = getToolByName(self.context, 'portal_properties', None)
        openmember_properties = getattr(portal_properties, 'openmember_properties', None)
        if openmember_properties.hasProperty('address_fields'):
            return list(openmember_properties.getProperty('address_fields'))
        else:
            return []

    def getFields(self):
        #FIXME: should be a property
        #field_order = self.openmember_properties.fields_to_index
        field_order = self.omindexes
        catalog_indexes = self.openmember_catalog.getIndexObjects()
        #FIXME: title should be field label info.
        result = []
        #Sort according to field order in portal_properties
        for index in field_order:
            for field in catalog_indexes:
                if index == field.id:
                    result.append( {'id':field.id,
                                    'type':field.meta_type,
                                    'title':getattr(field,'label',field.id)})
        return result

    def getFormatedField(self,item,field):
        """Hook to format specific fields. Right now only used for DateIndex"""
        
        #Fetch data
        data = getattr(item,field['id'])

        #Make dates cute
        if field['type'] == 'DateIndex':
            return self.context.toLocalizedTime(data, long_format=0)
        #Fetch sub-items of membership_references or other lists and return as a list
        if type(data) == list:
            result = []
            for uid in data:
                if uid in self.referenced_objects.keys():
                    result.append(self.referenced_objects[uid])
            return result
        
        #Just return data
        return data

    def showGeoLinks(self):
        """Check if context has enough metadata to make a query to google maps"""
        fields_to_check = self.getAddressFields()
        # this '&' is the intersection between two sets, i.e results in any element that are both lists.
        if set(fields_to_check) & set(self.omindexes):
            return True
        return False
    
    def getGeoQuery(self, item):
        """ Construct a query for google maps lookup through a link
        """
        #Don't lookup if to few parameters are provided
        locationkeys = self.getAddressFields()

        query = ''
        for key in locationkeys:
            field = getattr(item, key, None)
            if field:
                query += "%s+" % quote_plus(field)
        #FIXME: Add a usable label
        #if 'label' in xxx:
        #    query += "(%s)" % label
        return query
        