from zope.interface import implements
from zope.component import adapts
from zope.formlib import form
from zope.event import notify

from plone.protect import CheckAuthenticator
from plone.fieldsets.form import FieldsetsEditForm
from plone.app.controlpanel.interfaces import IPloneControlPanelForm
from plone.app.controlpanel.widgets import MultiCheckBoxVocabularyWidget
from plone.app.controlpanel.events import ConfigurationChangedEvent

from AccessControl import getSecurityManager
from AccessControl.Permissions import view_management_screens
from Acquisition import aq_inner

from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFPlone.interfaces import IPloneSiteRoot
#from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

from betahaus.openmember import MessageFactory as _
from betahaus.openmember import CATALOG_ID
from betahaus.openmember.interfaces import IOMControlPanelSchema


class OMControlPanelAdapter(SchemaAdapterBase):
    """Control panel adapter"""
    implements(IOMControlPanelSchema)
    adapts(IPloneSiteRoot)

    def __init__(self, context):
        super(OMControlPanelAdapter, self).__init__(context)
        self.omcatalog = getToolByName(context, CATALOG_ID)
        self.pprop = getToolByName(context, 'portal_properties')
        self.default_charset = self.pprop.site_properties.default_charset
        #self.context = pprop.openmember_properties

#    def get_member_candidate_types(self):
#        return self.context.member_candidate_types
#
#    def set_member_candidate_types(self, value):
#        self.context.member_candidate_types = value
#
#    member_candidate_types = property(get_member_candidate_types,
#                                      set_member_candidate_types)

    def get_fields_to_index(self):
        result = []
        for item in self.omcatalog.getIndexObjects():
            label = getattr(item, 'label', None)
            if not label:
                label = item.id
                
            result.append("%s|%s|%s" % (item.id, item.meta_type, label.decode(self.default_charset)) )

        return result

    def set_fields_to_index(self, value):

        # add indexes and columns that are not in the catalog yet.
        # prune the omindex, omcolumns so we know if there are any unneeded indexes, columns
        indexes = []
        for row in value:
            values = row.strip().split("|")
            if len(values) == 3:
                name = values[0]
                meta_type = values[1]
                label = values[2]
                
            elif len(values) == 2:
                name = values[0]
                meta_type = values[1]
                label = values[0]
            else:
                raise ValueError('wrong number of values, must be 2 or 3.')
            
            indexes.append((name.encode(self.default_charset),meta_type.encode(self.default_charset), label.encode(self.default_charset)))
                    
        self.omcatalog.setMirrorFields(indexes)
        self.omcatalog.clearFindAndRebuild()

    fields_to_index = property(get_fields_to_index,
                               set_fields_to_index)
    
    def get_address_fields(self):
        """The address fields are stored in the properties tool, on the openmember_properties sheet lines field called address_fields
        """
        portal_properties = getToolByName(self.context, 'portal_properties', None)
        openmember_properties = getattr(portal_properties, 'openmember_properties', None)
        if openmember_properties.hasProperty('address_fields'):
            return list(openmember_properties.getProperty('address_fields'))
        else:
            return []
    
    def set_address_fields(self, value):
        """The address fields are stored in the properties tool, on the openmember_properties sheet lines field called address_fields.
        """
        portal_properties = getToolByName(self.context, 'portal_properties', None)
        openmember_properties = getattr(portal_properties, 'openmember_properties', None)
        openmember_properties.manage_changeProperties(address_fields=tuple(value))
    
    address_fields = property(get_address_fields,
                              set_address_fields)



class OMControlPanel(FieldsetsEditForm):

    implements(IPloneControlPanelForm)

    form_fields = form.FormFields(IOMControlPanelSchema)
    #form_fields['member_candidate_types'].custom_widget = MultiCheckBoxVocabularyWidget

    label = _(u"Open Member control panel")
    description = _(u"Tools and settings for Open Member database.")
    form_name = _(u"Form name")


    def available(self):
        root = aq_inner(self.context).getPhysicalRoot()
        sm = getSecurityManager()
        return sm.checkPermission(view_management_screens, root)
    
    @form.action(_(u'label_save', default=u'Save'), name=u'save')
    def handle_edit_action(self, action, data):
        CheckAuthenticator(self.request)
        if form.applyChanges(self.context, self.form_fields, data,
                             self.adapters):
            self.status = _("Changes saved.")
            notify(ConfigurationChangedEvent(self, data))
        else:
            self.status = _("No changes made.") #Does this work? No Idea?


    @form.action(_(u'Clear and rebuild catalog'), name=u'clear_and_rebuild')
    def clear_and_rebuild(self, action, data):
        CheckAuthenticator(self.request)
        if not self.available():
            self.status = _(u'text_not_allowed_manage_server',
                            default=u'You are not allowed to manage the Zope server.')        
            return
        catalog = getToolByName(self.context, 'openmember_catalog')
        catalog.clearFindAndRebuild()
        self.status = _(u'Clear and rebuild complete')
        
